import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;

/**
 * PDFファイルのアーカイブ操作に関する操作の例
 * <p>
 * HTTPマルチパートフォームデータの送信例
 * </p>
 */
public class Archives {
    /* 文書アーカイブ用の URL */
    private static final String ARCHIVES_URL = "https://xxxxx.spa-cloud.com/spa/service/archives";

    /* 改行コード */
    private static final String CRLF = "\r\n";

    /* SVF PDF Archiver が返す独自のエラーコードの名称 */
    private static final String SPA_ERROR_CODE = "X-Spa-Error-Code";

    /**
     * 指定されたファイルのアーカイブを実行します。
     * 
     * <p>
     * multipart/form-dataでファイルをアップロードする例
     * </p>
     * 
     * @param cookies ログイン時に取得したすべてのcookie
     * @param xsrfToken XSRFトークン
     * @param path アップロード先のパス
     * @param name アップロード時のファイル名
     * @param file アップロードするファイル
     * @throws IOException サーバーへの接続に失敗した場合
     */
    public static void add(String cookies, String xsrfToken, String path, String name, File file) throws IOException {
        // HttpURLConnectionの作成
        URL url = new URL(ARCHIVES_URL);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);

            // すべてのcookieをセット
            conn.setRequestProperty("Cookie", cookies);
            // XSRF Tokenをセット
            conn.setRequestProperty("x-xsrf-token", xsrfToken);

            // マルチパートメッセージの作成
            String boundary = Long.toString(System.currentTimeMillis());
            conn.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
            conn.setRequestProperty("x-requested-with", "XMLHttpRequest");

            try (DataOutputStream dos = new DataOutputStream(new BufferedOutputStream(conn.getOutputStream()))) {
                writeFormData(dos, boundary, "path", path);
                writeFormData(dos, boundary, "name", name + boundary + ".pdf");
                writeFileData(dos, boundary, "file", file.getName(), file);
                dos.writeBytes("--" + boundary + "--" + CRLF);
            }

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // 独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt(SPA_ERROR_CODE, -1);
                System.out.println("ErrorCode[" + errCode + "]");
                throw new IOException(conn.getResponseMessage());
            }
            // 正常終了時
        } finally {
            conn.disconnect();
        }
    }

    /**
     * フォームデータの書き込み処理
     * 
     * @param dos DataOutputStream
     * @param boundary boundary文字列
     * @param name フィールド名
     * @param value 値
     * @throws IOException 書き込み処理に失敗した場合
     */
    private static void writeFormData(DataOutputStream dos, String boundary, String name, String value)
            throws IOException {
        dos.writeBytes("--" + boundary + CRLF);

        dos.writeBytes("Content-Disposition: form-data; name=\"" + name + "\"" + CRLF);
        dos.writeBytes(CRLF);
        dos.write(value.getBytes(StandardCharsets.UTF_8));
        dos.writeBytes(CRLF);
    }

    /**
     * フォームデータのファイルへの書き込み処理
     * 
     * @param dos DataoutputStream
     * @param boundary boundary文字列
     * @param name フィールド名
     * @param value 値
     * @param file 送信するファイル
     * @throws IOException 書き込み処理に失敗した場合
     */
    private static void writeFileData(DataOutputStream dos, String boundary, String name, String value, File file)
            throws IOException {

        dos.writeBytes("--" + boundary + CRLF);

        StringBuilder sb = new StringBuilder();
        sb.append("Content-Disposition: form-data; ");
        sb.append("name=\"").append(name).append("\"; ");
        sb.append("filename=\"").append(value).append("\"");
        sb.append(CRLF);
        dos.write(sb.toString().getBytes(StandardCharsets.UTF_8));

        dos.writeBytes("Content-Type: application/pdf" + CRLF);
        dos.writeBytes(CRLF);

        // ファイルの書き込み
        try (BufferedInputStream fis = new BufferedInputStream(new FileInputStream(file))) {
            byte[] buffer = new byte[4096];
            int len = -1;
            while ((len = fis.read(buffer)) != -1) {
                dos.write(buffer, 0, len);
            }
        }
    }

    /**
     * 指定したPDFファイルをアップロードします
     * 
     * @param args プログラム引数
     */
    public static void main(String[] args) {
        Authenticator auth = new Authenticator();
        try {
        	auth.login("admin", "xxxxx");
            if (auth.logined()) {
                File pdf = new File("sample.pdf");
                Archives.add(auth.getCookies(), auth.getXsrfToken(), "/Doc001", "sample", pdf);

                auth.logout();
            };
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
