import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * ユーザー認証（ログイン、ログアウト、パスワード変更）に関する操作の例
 * 
 * <p>
 * POST/PUTメソッドを使った例
 * </p>
 */
public class Authenticator {
    /* SVF PDF ArchiverのAPIの基本URL */
    private static final String BASE_URL = "https://xxxxx.spa-cloud.com/spa/service";
    /* ログイン用のURL */
    private static final String LOGIN_URL = BASE_URL + "/auth/login";
    /* ログアウト用のURL */
    private static final String LOGOUT_URL = BASE_URL + "/auth/logout";
    /* パスワード変更用のURL */
    private static final String PASSWORD_URL = BASE_URL + "/auth/password";

    /* SVF PDF Archiver が返す独自のエラーコードの名称 */
    private static final String SPA_ERROR_CODE = "X-Spa-Error-Code";

    /* ログイン状態 */
    private boolean logined = false;

    /* ログイン時に取得したすべてのクッキー */
    private String cookies = null;
    
    /* ログイン時に取得したXSRF Token*/
    private String xsrfToken;


    /**
     * 新しいインスタンスを生成します
     */
    public Authenticator() {
        this.cookies = null;
    }

    /**
     * ログイン処理
     * 
     * <p>
     * パラメーターを渡してPOSTメソッドを実行する例
     * </p>
     * 
     * @param user ユーザーID
     * @param password パスワード
     * @throws IOException サーバーへの接続に失敗した場合
     */
    public void login(String user, String password) throws IOException {
        // HttpURLConnectionの作成
        URL url = new URL(LOGIN_URL);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            // パラメーターで渡されたユーザーIDとパスワードをセットする
            try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), "UTF-8"))) {
                writer.write("user=" + user);
                writer.write("&password=" + password);
            }

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // 独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt(SPA_ERROR_CODE, -1);
                System.out.println("ErrorCode[" + errCode + "]");
                throw new IOException(conn.getResponseMessage());
                
            }

            // 正常終了時
            this.logined = true;
            System.out.println("login succeed.");

            // すべてのcookieを保持し、cookieからXSRFトークンを取り出しておく
            Map<String, List<String>> headers = conn.getHeaderFields();
        	StringBuffer sbCookie = new StringBuffer();
            for (Map.Entry<String, List<String>> entry:headers.entrySet()) {
            	String key = entry.getKey();
            	if (key != null && key.equals("Set-Cookie")) {
            		List<String> list = entry.getValue();
                    for(Iterator<String>itr = list.iterator(); itr.hasNext();){
                    	String cookie = itr.next();
                    	sbCookie.append(cookie).append("; ");
                        String[] array = cookie.split(";");
                        for (String str : array) {
                            if (str.matches("XSRF-TOKEN=.*")) {
                            	String[] kvp = str.split("=");
                            	this.xsrfToken = kvp[1];
                            	System.out.println("XSRF-TOKEN=" + this.xsrfToken);
                            	break;
                            }
                        }
                    }
                }
            }
            this.cookies = sbCookie.toString();
        } finally {
            // コネクションの切断
            conn.disconnect();
        }
    }

    /**
     * ログアウト処理
     * 
     * <p>
     * パラメーターを渡す必要のないPOSTメソッドを実行する例
     * </p>
     * 
     * @throws IOException サーバーへの接続に失敗した場合
     */
    public void logout() throws IOException {
        // HttpURLConnectionの作成
        URL url = new URL(LOGOUT_URL);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);

            // ログイン時に取得したすべてのcookieとXSRFトークンをセットする
            conn.setRequestProperty("Cookie", this.cookies);
            conn.setRequestProperty("x-xsrf-token", xsrfToken);
            conn.setRequestProperty("x-requested-with", "XMLHttpRequest");

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // 独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt(SPA_ERROR_CODE, -1);
                System.out.println("ErrorCode[" + errCode + "]");
                throw new IOException(conn.getResponseMessage());
            }
            System.out.println("logout succeed.");
        } finally {
            // コネクションの切断
            conn.disconnect();
            this.cookies = null;
        }
    }

    /**
     * ログイン状態を返します
     * 
     * @return ログイン状態
     */
    public boolean logined() {
        return this.logined;
    }

    /**
     * ログイン時に取得したすべてのcookieを返します
     * 
     * @return ログイン時に取得したすべてのcookie
     */
    public String getCookies() {
        return this.cookies;
    }

    /**
     * ログイン時に取得したXSRF Tokenを返します
     * 
     * @return ログイン時に取得したXSRF Token
     */
    public String getXsrfToken() {
        return this.xsrfToken;
    }

    /**
     * ログイン中のユーザーパスワードの変更処理
     * 
     * <p>
     * パラメーターを渡してPUTメソッドを実行する例
     * </p>
     * 
     * @param password 現在のパスワード
     * @param newPassword 新しいパスワード
     * @throws IOException サーバーへの接続に失敗した場合
     */
    public void changePassword(String password, String newPassword) throws IOException {
        // HttpURLConnectionの作成
        URL url = new URL(PASSWORD_URL);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("PUT");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            // ログイン時に取得したすべてのcookieとXSRFトークンをセットする
            conn.setRequestProperty("Cookie", this.cookies);
            conn.setRequestProperty("x-xsrf-token", this.xsrfToken);
            conn.setRequestProperty("x-requested-with", "XMLHttpRequest");

            // パラメーターで渡された新旧のパスワードをセットする
            try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), "UTF-8"))) {
                writer.write("password=" + password);
                writer.write("&newPassword=" + newPassword);
            }

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // 独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt(SPA_ERROR_CODE, -1);
                System.out.println("ErrorCode[" + errCode + "]");
                throw new IOException(conn.getResponseMessage());
            }
        } finally {
            // コネクションの切断
            conn.disconnect();
        }
    }

    /**
     * SVF PDF ArchiverへAPIを使ってログインしてからログアウトします
     * 
     * @param args プログラム引数
     */
    public static void main(String[] args) {
        Authenticator auth = new Authenticator();
        try {
        	auth.login(/*id*/"admin", /*password*/"xxxxx");
            if (auth.getCookies() != null && auth.getXsrfToken() != null) {
                // パスワードを変更する場合の例
                //auth.changePassword("xxxxx", "yyyyy");
                auth.logout();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
